package com.sun.org.apache.xpath.internal;

import com.sun.org.apache.xpath.internal.res.XPATHMessages;
import com.sun.org.apache.xml.internal.dtm.DTM;
import com.sun.org.apache.xml.internal.dtm.DTMFilter;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.NodeIterator;
import com.sun.org.apache.xml.internal.dtm.DTMManager;
import com.sun.org.apache.xml.internal.dtm.DTMIterator;
import com.sun.org.apache.xml.internal.utils.NodeVector;

public class NodeSetDTM extends NodeVector implements DTMIterator, Cloneable
{
    static final long serialVersionUID = 7686480133331317070L;
    DTMManager m_manager;
    protected transient int m_next;
    protected transient boolean m_mutable;
    protected transient boolean m_cacheNodes;
    protected int m_root;
    private transient int m_last;
    
    public NodeSetDTM(DTMManager dtmManager) {
        this.m_next = 0;
        this.m_mutable = true;
        this.m_cacheNodes = true;
        this.m_root = -1;
        this.m_last = 0;
        this.m_manager = dtmManager;
    }
    
    public NodeSetDTM(int blocksize, int dummy, DTMManager dtmManager) {
        super(blocksize);
        this.m_next = 0;
        this.m_mutable = true;
        this.m_cacheNodes = true;
        this.m_root = -1;
        this.m_last = 0;
        this.m_manager = dtmManager;
    }
    
    public NodeSetDTM(NodeSetDTM nodelist) {
        this.m_next = 0;
        this.m_mutable = true;
        this.m_cacheNodes = true;
        this.m_root = -1;
        this.m_last = 0;
        this.m_manager = nodelist.getDTMManager();
        this.m_root = nodelist.getRoot();
        this.addNodes(nodelist);
    }
    
    public NodeSetDTM(DTMIterator ni) {
        this.m_next = 0;
        this.m_mutable = true;
        this.m_cacheNodes = true;
        this.m_root = -1;
        this.m_last = 0;
        this.m_manager = ni.getDTMManager();
        this.m_root = ni.getRoot();
        this.addNodes(ni);
    }
    
    public NodeSetDTM(NodeIterator iterator, XPathContext xctxt) {
        this.m_next = 0;
        this.m_mutable = true;
        this.m_cacheNodes = true;
        this.m_root = -1;
        this.m_last = 0;
        this.m_manager = xctxt.getDTMManager();
        Node node;
        while (null != (node = iterator.nextNode())) {
            int handle = xctxt.getDTMHandleFromNode(node);
            this.addNodeInDocOrder(handle, xctxt);
        }
    }
    
    public NodeSetDTM(NodeList nodeList, XPathContext xctxt) {
        this.m_next = 0;
        this.m_mutable = true;
        this.m_cacheNodes = true;
        this.m_root = -1;
        this.m_last = 0;
        this.m_manager = xctxt.getDTMManager();
        for (int n = nodeList.getLength(), i = 0; i < n; ++i) {
            Node node = nodeList.item(i);
            int handle = xctxt.getDTMHandleFromNode(node);
            this.addNode(handle);
        }
    }
    
    public NodeSetDTM(int node, DTMManager dtmManager) {
        this.m_next = 0;
        this.m_mutable = true;
        this.m_cacheNodes = true;
        this.m_root = -1;
        this.m_last = 0;
        this.m_manager = dtmManager;
        this.addNode(node);
    }
    
    public void setEnvironment(Object environment) {
    }
    
    @Override
    public int getRoot() {
        if (-1 == this.m_root) {
            if (this.size() > 0) {
                return this.item(0);
            }
            return -1;
        }
        else {
            return this.m_root;
        }
    }
    
    @Override
    public void setRoot(int context, Object environment) {
    }
    
    @Override
    public Object clone() throws CloneNotSupportedException {
        NodeSetDTM clone = (NodeSetDTM)super.clone();
        return clone;
    }
    
    @Override
    public DTMIterator cloneWithReset() throws CloneNotSupportedException {
        NodeSetDTM clone = (NodeSetDTM)this.clone();
        clone.reset();
        return clone;
    }
    
    @Override
    public void reset() {
        this.m_next = 0;
    }
    
    @Override
    public int getWhatToShow() {
        return -17;
    }
    
    public DTMFilter getFilter() {
        return null;
    }
    
    @Override
    public boolean getExpandEntityReferences() {
        return true;
    }
    
    @Override
    public DTM getDTM(int nodeHandle) {
        return this.m_manager.getDTM(nodeHandle);
    }
    
    @Override
    public DTMManager getDTMManager() {
        return this.m_manager;
    }
    
    @Override
    public int nextNode() {
        if (this.m_next < this.size()) {
            int next = this.elementAt(this.m_next);
            ++this.m_next;
            return next;
        }
        return -1;
    }
    
    @Override
    public int previousNode() {
        if (!this.m_cacheNodes) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_CANNOT_ITERATE", (Object[])null));
        }
        if (this.m_next - 1 > 0) {
            --this.m_next;
            return this.elementAt(this.m_next);
        }
        return -1;
    }
    
    @Override
    public void detach() {
    }
    
    @Override
    public void allowDetachToRelease(boolean allowRelease) {
    }
    
    @Override
    public boolean isFresh() {
        return this.m_next == 0;
    }
    
    @Override
    public void runTo(int index) {
        if (!this.m_cacheNodes) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_CANNOT_INDEX", (Object[])null));
        }
        if (index >= 0 && this.m_next < this.m_firstFree) {
            this.m_next = index;
        }
        else {
            this.m_next = this.m_firstFree - 1;
        }
        return;
    }
    
    @Override
    public int item(int index) {
        this.runTo(index);
        return this.elementAt(index);
    }
    
    @Override
    public int getLength() {
        this.runTo(-1);
        return this.size();
    }
    
    public void addNode(int n) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        this.addElement(n);
        return;
    }
    
    public void insertNode(int n, int pos) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        this.insertElementAt(n, pos);
        return;
    }
    
    public void removeNode(int n) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        this.removeElement(n);
        return;
    }
    
    public void addNodes(DTMIterator iterator) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        if (null != iterator) {
            int obj;
            while (-1 != (obj = iterator.nextNode())) {
                this.addElement(obj);
            }
        }
        return;
    }
    
    public void addNodesInDocOrder(DTMIterator iterator, XPathContext support) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        int node;
        while (-1 != (node = iterator.nextNode())) {
            this.addNodeInDocOrder(node, support);
        }
        return;
    }
    
    public int addNodeInDocOrder(int node, boolean test, XPathContext support) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        int insertIndex = -1;
        if (test) {
            int size = this.size();
            int i = size - 1;
            while (i >= 0) {
                int child = this.elementAt(i);
                if (child == node) {
                    i = -2;
                    break;
                }
                DTM dtm = support.getDTM(node);
                if (!dtm.isNodeAfter(node, child)) {
                    break;
                }
                --i;
            }
            if (i != -2) {
                insertIndex = i + 1;
                this.insertElementAt(node, insertIndex);
            }
        }
        else {
            insertIndex = this.size();
            boolean foundit = false;
            int i = 0;
            while (i < insertIndex) {
                if (i == node) {
                    foundit = true;
                    break;
                }
                ++i;
            }
            if (!foundit) {
                this.addElement(node);
            }
        }
        return insertIndex;
    }
    
    public int addNodeInDocOrder(int node, XPathContext support) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        return this.addNodeInDocOrder(node, true, support);
    }
    
    @Override
    public int size() {
        return super.size();
    }
    
    @Override
    public void addElement(int value) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        super.addElement(value);
        return;
    }
    
    @Override
    public void insertElementAt(int value, int at) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        super.insertElementAt(value, at);
        return;
    }
    
    @Override
    public void appendNodes(NodeVector nodes) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        super.appendNodes(nodes);
        return;
    }
    
    @Override
    public void removeAllElements() {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        super.removeAllElements();
        return;
    }
    
    @Override
    public boolean removeElement(int s) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        return super.removeElement(s);
    }
    
    @Override
    public void removeElementAt(int i) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        super.removeElementAt(i);
        return;
    }
    
    @Override
    public void setElementAt(int node, int index) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        super.setElementAt(node, index);
        return;
    }
    
    @Override
    public void setItem(int node, int index) {
        if (!this.m_mutable) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_NOT_MUTABLE", (Object[])null));
        }
        super.setElementAt(node, index);
        return;
    }
    
    @Override
    public int elementAt(int i) {
        this.runTo(i);
        return super.elementAt(i);
    }
    
    @Override
    public boolean contains(int s) {
        this.runTo(-1);
        return super.contains(s);
    }
    
    @Override
    public int indexOf(int elem, int index) {
        this.runTo(-1);
        return super.indexOf(elem, index);
    }
    
    @Override
    public int indexOf(int elem) {
        this.runTo(-1);
        return super.indexOf(elem);
    }
    
    @Override
    public int getCurrentPos() {
        return this.m_next;
    }
    
    @Override
    public void setCurrentPos(int i) {
        if (!this.m_cacheNodes) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_NODESETDTM_CANNOT_INDEX", (Object[])null));
        }
        this.m_next = i;
        return;
    }
    
    @Override
    public int getCurrentNode() {
        if (!this.m_cacheNodes) {
            throw new RuntimeException("This NodeSetDTM can not do indexing or counting functions!");
        }
        int saved = this.m_next;
        int current = (this.m_next > 0) ? (this.m_next - 1) : this.m_next;
        int n = (current < this.m_firstFree) ? this.elementAt(current) : -1;
        this.m_next = saved;
        return n;
    }
    
    public boolean getShouldCacheNodes() {
        return this.m_cacheNodes;
    }
    
    @Override
    public void setShouldCacheNodes(boolean b) {
        if (!this.isFresh()) {
            throw new RuntimeException(XPATHMessages.createXPATHMessage("ER_CANNOT_CALL_SETSHOULDCACHENODE", (Object[])null));
        }
        this.m_cacheNodes = b;
        this.m_mutable = true;
        return;
    }
    
    @Override
    public boolean isMutable() {
        return this.m_mutable;
    }
    
    public int getLast() {
        return this.m_last;
    }
    
    public void setLast(int last) {
        this.m_last = last;
    }
    
    @Override
    public boolean isDocOrdered() {
        return true;
    }
    
    @Override
    public int getAxis() {
        return -1;
    }
}
